defmodule Chronoscope.Certificate do
  @date_time Application.compile_env(:chronoscope, :date_time, DateTime)

  def expiration_date(certificate) do
    case(
      certificate
      |> X509.Certificate.from_der!()
      |> X509.Certificate.validity()
    ) do
      {:Validity, _, {:utcTime, expiration}} ->
        cert_time_to_iso8601(expiration)

      {:Validity, {:utcTime, expiration}, _} ->
        cert_time_to_iso8601(expiration)
    end
  end

  def cert_time_to_iso8601(cert_time) do
    captures =
      Regex.named_captures(
        ~r/^(?<year>\d\d)(?<month>\d\d)(?<day>\d\d)(?<hour>\d\d)(?<minute>\d\d)(?<second>\d\d)Z$/,
        to_string(cert_time)
      )

    "#{short_year_to_full_year(captures["year"])}-#{captures["month"]}-#{captures["day"]}T#{captures["hour"]}:#{captures["minute"]}:#{captures["second"]}Z"
  end

  defp short_year_to_full_year(short_year) do
    {century, current_year} =
      @date_time.utc_now().year
      |> to_string()
      |> String.split_at(-2)

    if String.to_integer(short_year) >= String.to_integer(current_year) do
      century <> short_year
    else
      century
      |> String.to_integer()
      |> then(&"#{&1 + 1}#{short_year}")
    end
  end
end
